#ifndef _BSERVER_H_
#define _BSERVER_H_

/*
   *
   * bserver.h
   *
   * Copyright (c) 2002-2003 Aleksander Augustowski
   *
   * BUG RAPORTS &
   * E-MAIL:          aleksander.augustowski@ins.com.pl
   * URL:             http://www.ins.com.pl/~alex/bcount/bcount.html
   *
   * This software is open source; you can redistribute it and/or modify
   * it under the terms of the GNU General Public License as published by
   * the Free Software Foundation; either version 2 of the License, or
   * (at your option) any later version.
   *
   * This program is distributed WITHOUT ANY WARRANTY; without even the
   * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
   * See the GNU General Public License in the included COPYING file for
   * details.
   *
   *
*/


#include <sys/wait.h>
#include <curses.h>
#include <fcntl.h>
#include "programs_version.h"
#include "includes.h"
#include "constans.h"
#include "protocol.h"
#include "bugy_function.h"
#include "colours.h"
#include "colours_curses.h"
#include "xmalloc.h"
#include "host_info.h"
#include "author_info.h"
#include "print_info.h"
#include "data_type.h"
#include "sys_info.h"
#include "host_name.h"
#include "interface.h"
#include "protocol_version.h"
#include "server_port.h"
#include "date_time.h"
#include "smtp_client.h"
#include "terminal_setting.h"


/* Server name */
#define SNAME		"Bytes Counter Server"	

/* Default refresh time in sec. */
#define REFRESH		8

/* Connection amount than will be queued */
#define CLIENT_QUEUE	10


void port_error(void) __attribute__ ((noreturn));
void refresh_error(void) __attribute__ ((noreturn));
void verbose_error(void) __attribute__ ((noreturn));
void print_server_usage(void) __attribute__ ((noreturn));
int touching_bytes_data(char *dev);
void print_http_verbose1(void);
void print_http_verbose2(void);
void print_bcnt_verbose1(void);
void print_bcnt_verbose2(void);
int handle_get (int client_socket, const char* what_want);
int handle_connection (int client_socket);
int server_run(void);


struct option longopts[] = 
{
	{"address",         1, 0, 'a'},
	{"interface",       1, 0, 'i'},
	{"port",            1, 0, 'p'},
	{"smtp",            1, 0, 's'},
	{"refresh",         1, 0, 'r'},
	{"version",         0, 0, 'v'},
	{"verbose",         1, 0, 'm'},
	{"license",         0, 0, 'l'},
	{"help",            0, 0, 'h'},
	{ 0, 0, 0, 0 }
};


char header_buffer[5120], body_buffer[10240];
/*char received_buffer[5120];*/ /* This buffer declared in smtp_client.h */
char method[10], protocol[30];
char user_agent[200];
char cmessage[150], smessage[150] = " ";
char ipv4_address[15];
char *interface_bind_address = ipv4_address;
int verbose = 0, verbose_level = 1;
int resolve = 1, resolve_ok = 1;
int xrefresh = REFRESH;
int server_socket = -1, client_socket = -1;
int server_port = SPORT;
int interface_bind_address_flag = FALSE;
unsigned long hex_address;
unsigned long how_many_connections = 0;



/* Procedure to serve SIGCHLD signal. Cleanning ending descendant process */
void clean_up_child_process (int signal_number)
{
	int status;
	wait (&status);
}


void port_error(void)
{
	printf ("Server port "_STRING_WHITE_" is not valid,"
		" exiting...\n", optarg);

	exit_prog(EXIT_FALSE);
}


void interface_address_error(void)
{
	printf ("Interface address "_STRING_WHITE_" is not valid,"
		" exiting...\n", interface_bind_address);

	exit_prog(EXIT_FALSE);
}


void print_server_version(void)
{
	printf ("BYTES COUNTER SERVER v"_STRING_WHITE_, SVERSION);
}


void print_welcome(void)
{
	printf ("-== ");
	print_server_version();
	printf (" on an %s running... ==-\n"
		"Copyright (c) 2002-2003 %s\n%s\n\n",
		get_local_host_info(HOST_ARCHITECTURE),
		email_address, page_address);
	
	free(u_name);
}


void refresh_error(void)
{
	printf ("Please give me only digit. "
		_STRING_WHITE_" is alpha.\n", optarg);

	exit_prog(EXIT_FALSE);
}


void verbose_error(void)
{
	printf ("Please give me only digit. "
		"Verbose Level for now "
		_COLOR_WHITE_"1"_COLOR_END_" or "
		_COLOR_WHITE_"2"_COLOR_END_".\n");

	exit_prog(EXIT_FALSE);
}


void print_server_usage(void)
{
	printf ("Usage: %s [OPTIONS] ...", exec_name);
	printf ("\nExample: %s -i ippp0  <- Choice ippp0 as default interface\n"
		"Options:\n"
		"  -a [ADDRESS]   --address       "
		"Choice interface ip address to get requests\n                 "
		"                (default: get from any address)\n"
		"  -i [INTERFACE] --interface     "
		"Choice Interface (default: %s)\n"
		"  -p [PORT]      --port          "
		"Choice server port (default: %d)\n"
		"  -n                             "
		"Don't resolve address (default: resolve is on)\n"
		"  -s [SERVER]    --smtp          "
		"Remote SMTP server name.\n"
		"  -r [INTEGER]   --refresh       "
		"Time (sec.) until WWW Client refresh info\n"
		"  -m [LEVEL]     --verbose       "
		"Print what Server send to client (Level: 1, 2)\n"
		"  -g      	 --signals       See information about signals\n"
		"  -l             --license       "
		"Print license for this program\n"
		"  -v, -V         --version       Print server version\n"
		"  -h             --help          Print short help\n\n"
		"     Hst -> Connected Host\n"
		"     Clt -> Client name\n"
		"     Prl -> Client protocol name & version\n"
		"     Mtd -> Client method (ex.: GET)\n"
		"     Get -> What client want from server\n"
		"     Msg -> Information from client\n"
		"     BE  -> Big endian (Network byte order)\n\n", 
		exec_name, default_interface_name, SPORT);

	exit_prog(EXIT_SUCCESS);
}



/* NIN */

#endif /* _BSERVER_H_ */

